package cz.fi.muni.pa165.entity;

import cz.fi.muni.pa165.enums.BandType;
import lombok.Getter;
import lombok.Setter;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

/**
 * @author Dávid Želník - 469323
 */
@Entity
@Getter
@Setter
public class Band {

    @Id
    @NotNull
    private String id = UUID.randomUUID().toString();

    @NotNull
    private String name;

    @NotNull
    @Enumerated(EnumType.STRING)
    private BandType bandType;

    @Column
    private String logo;

    @OneToMany
    private Set<Activity> activities = new HashSet<>();

    @OneToMany(mappedBy = "band")
    private Set<Album> bandAlbums = new HashSet<>();

    @NotNull
    @OneToMany(mappedBy = "band")
    private Set<Musician> bandMembers = new HashSet<>();

    public Band() {}

    @Override
    public boolean equals(Object o) {
        if (o == null)
            return false;
        if (this == o)
            return true;
        if (!(o instanceof Band))
            return false;
        Band band = (Band) o;
        return getId().equals(band.getId());
    }

    @Override
    public int hashCode() {
        final int prime = 71;
        return prime * (getId() != null ? getId().hashCode() : 0);
    }

    public void addMusician(Musician musician){
        bandMembers.add(musician);
    }

    public void removeMusician(Musician musician){
        bandMembers.remove(musician);
    }

    public void addAlbum(Album album) {
        bandAlbums.add(album);
    }

    public void addActivity(Activity activity){
        activities.add(activity);
    }

    @Override
    public String toString() {
        return "Band{" +
                "id=" + id +
                ", name='" + name + '\'' +
                ", type=" + bandType +
                ", logo='" + logo + '\'' +
                ", bandAlbums=" + bandAlbums +
                ", bandMembers=" + bandMembers +
                '}';
    }
}
