package cz.fi.muni.pa165;


import cz.fi.muni.pa165.entity.*;
import cz.fi.muni.pa165.enums.BandType;
import cz.fi.muni.pa165.enums.MusicianType;
import cz.fi.muni.pa165.enums.UserRole;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Component
public class DataLoader implements ApplicationRunner {

    private final MusicianService musicianService;
    private final OfferService offerService;
    private final BandService bandService;
    private final UserService userService;
    private final ActivityService activityService;
    private final ManagerService managerService;
    private final SongService songService;
    private final AlbumService albumService;

    @Autowired
    public DataLoader(MusicianService musicianService, OfferService offerService, BandService bandService,
                      UserService userService, ActivityService activityService, ManagerService managerService,
                      SongService songService, AlbumService albumService) {
        this.musicianService = musicianService;
        this.offerService = offerService;
        this.bandService = bandService;
        this.userService = userService;
        this.activityService = activityService;
        this.managerService = managerService;
        this.songService = songService;
        this.albumService = albumService;
    }

    @Override
    @Transactional
    public void run(ApplicationArguments args) {
        createMusicians();
        createBand();
        createActivity();
        createUser();
        createSongs();
        createAlbum();
    }

    public void createMusicians(){
        Musician musicianEric = new Musician();
        musicianEric.setId("0");
        musicianEric.setType(MusicianType.GUITARIST);
        musicianEric.setName("Eric Stone");
        musicianService.create(musicianEric);

        Musician musicianLea = new Musician();
        musicianLea.setName("Lea Lee");
        musicianLea.setType(MusicianType.SINGER);
        musicianLea.setId("1");
        musicianService.create(musicianLea);

        Musician musicianTadeas = new Musician();
        musicianTadeas.setName("Tadeas Smith");
        musicianTadeas.setType(MusicianType.DRUMMER);
        musicianTadeas.setId("2");
        musicianService.create(musicianTadeas);

        Musician musicianRebecca = new Musician();
        musicianRebecca.setName("Rebecca Fischer");
        musicianRebecca.setType(MusicianType.PIANIST);
        musicianRebecca.setId("3");
        musicianService.create(musicianRebecca);

        Musician musicianMatt = new Musician();
        musicianMatt.setName("Matthew Tuck");
        musicianMatt.setType(MusicianType.SINGER);
        musicianMatt.setId("4");
        musicianService.create(musicianMatt);

        Musician musicianMichael = new Musician();
        musicianMichael.setName("Michael Paget");
        musicianMichael.setType(MusicianType.GUITARIST);
        musicianMichael.setId("5");
        musicianService.create(musicianMichael);

        Musician musicianJamie = new Musician();
        musicianJamie.setName("Jamie Mathias");
        musicianJamie.setType(MusicianType.BASS_GUITARIST);
        musicianJamie.setId("6");
        musicianService.create(musicianJamie);

        Musician musicianJason = new Musician();
        musicianJason.setName("Jason Bowld");
        musicianJason.setType(MusicianType.DRUMMER);
        musicianJason.setId("7");
        musicianService.create(musicianJason);

        Musician uno = new Musician();
        uno.setName("Uno");
        uno.setType(MusicianType.DRUMMER);
        uno.setId("8");
        musicianService.create(uno);

        Musician dos = new Musician();
        dos.setName("Dos");
        dos.setType(MusicianType.DRUMMER);
        dos.setId("9");
        musicianService.create(dos);

        Musician tres = new Musician();
        tres.setName("Tres");
        tres.setType(MusicianType.DRUMMER);
        tres.setId("10");
        musicianService.create(musicianJason);
    }

    public void createBand(){
        Band bandBFMV  = new Band();
        bandBFMV.setId("11");
        bandBFMV.setName("Bullet For My Valentine");
        bandBFMV.setBandType(BandType.METAL);
        bandBFMV.setLogo("BFMV");

        Musician matt = musicianService.findById("4");
        if (matt == null){
            throw new AssertionError();
        }

        Musician michael = musicianService.findById("5");
        if (michael == null){
            throw new AssertionError();
        }

        Musician jamie = musicianService.findById("6");
        if (jamie == null){
            throw new AssertionError();
        }
        Musician jason = musicianService.findById("7");
        if (jason == null){
            throw new AssertionError();
        }

        bandBFMV.setBandMembers(new HashSet<>(List.of(matt, jamie, jason, michael)));
        matt.setHasBand(true);
        matt.setBand(bandBFMV);
        jamie.setHasBand(true);
        jamie.setBand(bandBFMV);
        jason.setHasBand(true);
        jason.setBand(bandBFMV);
        michael.setHasBand(true);
        michael.setBand(bandBFMV);

        bandService.create(bandBFMV);

        musicianService.update(matt);
        musicianService.update(jamie);
        musicianService.update(michael);
        musicianService.update(jason);

        Band bottleOfWatter = new Band();
        bottleOfWatter.setName("Bottle of Watter");
        bottleOfWatter.setBandType(BandType.PUNK);
        bottleOfWatter.setId("12");
        bottleOfWatter.setLogo("bow");
        bandService.create(bottleOfWatter);

        Offer offer = new Offer();
        offer.setId("13");
        offer.setBand(bottleOfWatter);
        offer.setName("offer");
        offerService.create(offer);

        Band mexicanos = new Band();
        mexicanos.setName("Los Mexicanos");
        mexicanos.setBandType(BandType.COUNTRY);
        mexicanos.setId("14");
        mexicanos.setLogo("mx");
        Musician uno = musicianService.findById("8");
        Musician dos = musicianService.findById("9");
        uno.setBand(mexicanos);
        dos.setBand(mexicanos);
        mexicanos.setBandMembers(new HashSet<>(List.of(uno, dos)));
        bandService.create(mexicanos);
        uno.setHasBand(true);
        dos.setHasBand(true);
        musicianService.update(uno);
        musicianService.update(dos);

        Offer offer2 = new Offer();
        offer2.setId("15");
        offer2.setBand(mexicanos);
        offer2.setName("offer de mexico");
        offerService.create(offer2);
    }


    public void createUser(){
        User user = new User();
        user.setName("user");
        user.setPassword("user");
        user.setUserRole(UserRole.MUSICIAN);
        Musician jamie = musicianService.findById("6");
        user.setMusician(jamie);
        userService.create(user);

        Manager manager = new Manager();
        manager.setId("16");
        manager.setName("Imrich");
        managerService.create(manager);

        User userManager = new User();
        userManager.setName("manager");
        userManager.setPassword("manager");
        userManager.setUserRole(UserRole.MANAGER);
        userManager.setManager(manager);
        userService.create(userManager);
    }

    public void createActivity(){
        Activity activity = new Activity();
        activity.setName("Rock for People");
        Band bfmv = bandService.findById("11");
        if (bfmv == null){
            throw new AssertionError();
        }
        activity.setBand(bfmv);
        activity.setId("50");
        activity.setTime(LocalDateTime.of(2022, 6, 15, 0, 0));
        activityService.create(activity);

        Activity activity1 = new Activity();
        activity1.setTime(LocalDateTime.of(2022, 7,5, 0,0));
        activity1.setName("Concert");
        Band mv = bandService.findById("12");
        if (mv == null){
            throw new AssertionError();
        }
        activity1.setBand(mv);
        activity1.setId("51");
        activityService.create(activity1);
    }

    public void createSongs(){
        Song tears = new Song();
        tears.setSongName("Tears Don't Fall");
        tears.setDuration(Duration.ofMinutes(6));
        tears.setId("17");
        songService.create(tears);

        Song betrayal = new Song();
        betrayal.setSongName("Your Betrayal");
        betrayal.setDuration(Duration.ofMinutes(5));
        betrayal.setId("18");
        songService.create(betrayal);

        Song letting = new Song();
        letting.setSongName("Letting You Go");
        letting.setDuration(Duration.ofMinutes(4));
        letting.setId("19");
        songService.create(letting);

        Song hearts = new Song();
        hearts.setSongName("Hearts Burst into Fire");
        hearts.setDuration(Duration.ofMinutes(5));
        hearts.setId("20");
        songService.create(hearts);

        Song breathe = new Song();
        breathe.setSongName("Breathe Underwater");
        breathe.setDuration(Duration.ofMinutes(4));
        breathe.setId("21");
        songService.create(breathe);

        Song burrito = new Song();
        burrito.setSongName("Burrito");
        burrito.setDuration(Duration.ofMinutes(5));
        burrito.setId("22");
        songService.create(burrito);

        Song tacos = new Song();
        tacos.setSongName("Tacos");
        tacos.setDuration(Duration.ofMinutes(4));
        tacos.setId("23");
        songService.create(tacos);
    }


    public void createAlbum(){
        Album bestOf = new Album();
        Song tears = songService.findById("17");
        if (tears == null){
            throw new AssertionError();
        }
        Song betrayal = songService.findById("18");
        if (betrayal == null){
            throw new AssertionError();
        }
        Song letting = songService.findById("19");
        if (letting == null){
            throw new AssertionError();
        }
        Song hearts = songService.findById("20");
        if (hearts == null){
            throw new AssertionError();

        }
        Song breathe = songService.findById("21");
        if (breathe == null){
            throw new AssertionError();
        }
        bestOf.setName("Best of BFMV");
        Set<Song> songs = new HashSet<>(List.of(tears, betrayal, letting, hearts, breathe));
        bestOf.setSongs(songs);
        albumService.create(bestOf);

        Band bftv = bandService.findById("11");
        if (bftv == null){
            throw new AssertionError();
        }
        bestOf.setBand(bftv);
        songs.forEach(song -> song.setAlbum(bestOf));
        bandService.update(bftv);

        Album lasDias = new Album();
        Song burrito = songService.findById("22");
        Song tacos = songService.findById("23");

        lasDias.setName("Las Dias");
        Set<Song> songsM = new HashSet<>(List.of(burrito, tacos));
        lasDias.setSongs(songsM);
        burrito.setAlbum(lasDias);
        tacos.setAlbum(lasDias);
        albumService.create(lasDias);

        Band mexicanos = bandService.findById("14");
        lasDias.setBand(mexicanos);
        bandService.update(mexicanos);
    }

}
