package cz.fi.muni.pa165;


import cz.fi.muni.pa165.dto.user.UserDto;
import cz.fi.muni.pa165.mapper.DtoMapper;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@Service
public class SecurityUserDetail implements UserDetailsService {

    private final UserService userService;

    public SecurityUserDetail(UserService userService) {

        this.userService = userService;
    }

    @Override
    public UserDetails loadUserByUsername(String name) throws UsernameNotFoundException {
        var users = userService.findByName(name).stream().map(DtoMapper::mapToDto).collect(Collectors.toList());
        if (users.isEmpty()) {
            throw new UsernameNotFoundException("User does not exist");
        }
        var user = users.get(0);
        return new User(user.getName(), user.getPassword(), List.of(setRole(user)));
    }

    GrantedAuthority setRole(UserDto user) {
        return new SimpleGrantedAuthority("ROLE_" + user.getUserRole());
    }
}