package cz.fi.muni.pa165.contollers;

import cz.fi.muni.pa165.AlbumService;
import cz.fi.muni.pa165.BandService;
import cz.fi.muni.pa165.dto.album.AlbumCreateDto;
import cz.fi.muni.pa165.dto.album.AlbumDto;
import cz.fi.muni.pa165.dto.album.AlbumUpdateDto;
import cz.fi.muni.pa165.entity.Album;
import cz.fi.muni.pa165.entity.Band;
import cz.fi.muni.pa165.mapper.CreateDtoMapper;
import cz.fi.muni.pa165.mapper.DtoMapper;
import cz.fi.muni.pa165.mapper.UpdateDtoMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


import java.util.List;
import java.util.stream.Collectors;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

@CrossOrigin
@RestController
@RequestMapping("/album")
public class AlbumController {

    private AlbumService albumService;
    private BandService bandService;

    @Autowired
    public AlbumController(AlbumService albumService, BandService bandService){
        this.albumService = albumService;
        this.bandService = bandService;
    }

    @GetMapping
    public List<AlbumDto> getAlbums(){
        return albumService.findAll().stream().map(DtoMapper:: mapToDto).collect(Collectors.toList());
    }

    @PostMapping(value = "/create", consumes = APPLICATION_JSON_VALUE)
    public void create(@RequestBody AlbumCreateDto albumDto){
            Album album = CreateDtoMapper.mapToEntity(albumDto);
            Band band = bandService.findById(albumDto.getBandId());
            album.setBand(band);
            albumService.create(album);
            bandService.update(band);
    }

    @GetMapping(value = "/{id}", produces = APPLICATION_JSON_VALUE)
    public AlbumDto getAlbum(@PathVariable("id") long id) {
        return DtoMapper.mapToDto(albumService.findById(String.valueOf(id)));
    }

    @GetMapping(value = "/{name}", produces = APPLICATION_JSON_VALUE)
    public List<AlbumDto> getAlbum(@PathVariable("name") String name) {
        return albumService.findByName(name).stream().map(DtoMapper::mapToDto).collect(Collectors.toList());
    }

    @PutMapping(value = "/update", consumes = APPLICATION_JSON_VALUE)
    public void update(@RequestBody AlbumUpdateDto dto) {
        Album album = albumService.findById(dto.getId());
        Album updateAlbum = UpdateDtoMapper.mapToEntity(dto, album);
        updateAlbum.setBand(bandService.findById(dto.getBandId()));
        albumService.update(updateAlbum);
    }

    @DeleteMapping(value = "/delete", consumes = APPLICATION_JSON_VALUE)
    public void delete(@RequestBody AlbumUpdateDto dto) {
        Album album = albumService.findById(dto.getId());
        albumService.delete(album);
    }

    @GetMapping(value = "/albums_band/{id}", produces = APPLICATION_JSON_VALUE)
    public String getAlbumsBand(@PathVariable("id") String id){
        Album album = albumService.findById(id);
        return album.getBand().getName();
    }
}
