package cz.fi.muni.pa165.contollers;

import cz.fi.muni.pa165.ManagerService;
import cz.fi.muni.pa165.dto.band.BandDto;
import cz.fi.muni.pa165.dto.manager.ManagerCreateDto;
import cz.fi.muni.pa165.dto.manager.ManagerDto;
import cz.fi.muni.pa165.dto.manager.ManagerUpdateDto;
import cz.fi.muni.pa165.entity.Band;
import cz.fi.muni.pa165.entity.Manager;
import cz.fi.muni.pa165.mapper.CreateDtoMapper;
import cz.fi.muni.pa165.mapper.DtoMapper;
import cz.fi.muni.pa165.mapper.UpdateDtoMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

@CrossOrigin
@RestController
@RequestMapping("/manager")
public class ManagerController {

    private ManagerService managerService;

    @Autowired
    public ManagerController(ManagerService managerService) {
        this.managerService = managerService;
    }


    @GetMapping
    public List<ManagerDto> getManagers() {
        return managerService
                .findAll()
                .stream()
                .map(DtoMapper::mapToDto)
                .collect(Collectors.toList());
    }


    @GetMapping(value = "/{id}", produces = APPLICATION_JSON_VALUE)
    public ManagerDto getManager(@PathVariable("id") long id) {
        return DtoMapper.mapToDto(managerService.findById(String.valueOf(id)));
    }


    @PostMapping(value = "/create", consumes = APPLICATION_JSON_VALUE)
    public void create(@RequestBody ManagerCreateDto managerCreateDto) {
        managerService.create(CreateDtoMapper.mapToEntity(managerCreateDto));
    }

    @PutMapping(value = "/update", consumes = APPLICATION_JSON_VALUE)
    public void update(@Valid @RequestBody ManagerUpdateDto dto) {
        Manager manager = managerService.findById(dto.getId());
        Manager updatedManager = UpdateDtoMapper.mapToEntity(dto, manager);
        managerService.update(updatedManager);
    }

    @DeleteMapping(value = "/{id}")
    public void delete(@PathVariable String id) {
        Manager manager = managerService.findById(id);
        managerService.delete(manager);
    }


    @GetMapping(value = "/{id}/managed_bands", produces = APPLICATION_JSON_VALUE)
    public Set<BandDto> getManagedBands(@PathVariable("id") long id) {
        Manager manager = managerService.findById(String.valueOf(id));
        Set<Band> bands = managerService.findAllManagedBands(manager);
        return bands.stream().map(DtoMapper::mapToDto).collect(Collectors.toSet());
    }
}
