package cz.fi.muni.pa165.contollers;

import cz.fi.muni.pa165.BandService;
import cz.fi.muni.pa165.OfferService;
import cz.fi.muni.pa165.dto.band.BandDto;
import cz.fi.muni.pa165.entity.Band;
import cz.fi.muni.pa165.entity.Musician;
import cz.fi.muni.pa165.entity.Offer;
import cz.fi.muni.pa165.mapper.DtoMapper;
import cz.fi.muni.pa165.mapper.CreateDtoMapper;
import cz.fi.muni.pa165.MusicianService;
import cz.fi.muni.pa165.dto.musician.MusicianCreateDto;
import cz.fi.muni.pa165.dto.musician.MusicianDto;
import cz.fi.muni.pa165.dto.musician.MusicianUpdateDto;
import cz.fi.muni.pa165.mapper.UpdateDtoMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

import java.util.List;
import java.util.stream.Collectors;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;


@CrossOrigin
@RestController
@RequestMapping("/musicians")
public class MusicianController {

    private MusicianService musicianService;
    private BandService bandService;
    private OfferService offerService;

    @Autowired
    public MusicianController(MusicianService musicianService, BandService bandService, OfferService offerService) {
        this.musicianService = musicianService;
        this.bandService = bandService;
        this.offerService = offerService;
    }

    @GetMapping(value = "/{id}", produces = APPLICATION_JSON_VALUE)
    public MusicianDto getMusician(@PathVariable("id") long id) {
        return DtoMapper.mapToDto(musicianService.findById(String.valueOf(id)));
    }

    @PostMapping(value = "/create", consumes = APPLICATION_JSON_VALUE)
    public void create(@RequestBody MusicianCreateDto musicianDto) {
        Musician musician = CreateDtoMapper.mapToEntity(musicianDto);
        musicianService.create(musician);
    }

    @GetMapping(value = "/{id}/musicians_bands", produces = APPLICATION_JSON_VALUE)
    public BandDto getMusiciansBand(@PathVariable String id){
        Musician musician = musicianService.findById(id);
        List<Band> bands = bandService.findAll().stream().filter(band -> band.getBandMembers().contains(musician)).collect(Collectors.toList());
        return DtoMapper.mapToDto(bandService.findById(bands.get(0).getId()));
    }

    @PutMapping(value = "/{offerId}/accept_offer/{musicianId}", consumes = APPLICATION_JSON_VALUE)
    public void acceptOffer(@PathVariable("offerId") String offerId, @PathVariable("musicianId") String musicianId){
        Musician musician = musicianService.findById(musicianId);
        Offer offer = offerService.findById(offerId);
        bandService.acceptOffer(offer, musician);
        musicianService.update(musician);
        offerService.delete(offer);
    }

    @PutMapping(value = "/update", consumes = APPLICATION_JSON_VALUE)
    public void update(@Valid @RequestBody MusicianUpdateDto dto) {
        Musician musician = musicianService.findById(dto.getId());
        Musician updatedMusician = UpdateDtoMapper.mapToEntity(dto, musician);
        musicianService.update(updatedMusician);
    }

    @DeleteMapping(value = "/{id}")
    public void delete(@PathVariable String id) {
        Musician musician = musicianService.findById(id);
        musicianService.delete(musician);
    }

    @GetMapping
    public List<MusicianDto> getAvailableMusicians() {
        return musicianService.findAvailableMusicians().stream().map(DtoMapper::mapToDto).collect(Collectors.toList());
    }

}

