package cz.fi.muni.pa165.contollers;

import cz.fi.muni.pa165.BandService;
import cz.fi.muni.pa165.OfferService;
import cz.fi.muni.pa165.dto.offer.OfferCreateDto;
import cz.fi.muni.pa165.dto.offer.OfferDto;
import cz.fi.muni.pa165.dto.offer.OfferUpdateDto;
import cz.fi.muni.pa165.entity.Band;
import cz.fi.muni.pa165.entity.Offer;
import cz.fi.muni.pa165.mapper.CreateDtoMapper;
import cz.fi.muni.pa165.mapper.DtoMapper;
import cz.fi.muni.pa165.mapper.UpdateDtoMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.Collectors;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

@CrossOrigin
@RestController
@RequestMapping("/offer")
public class OfferController {
    private OfferService offerService;
    private BandService bandService;

    @Autowired
    public OfferController(OfferService offerService, BandService bandService) {
        this.offerService = offerService;
        this.bandService = bandService;
    }


    @GetMapping
    public List<OfferDto> getOffers() {
        var result = offerService.findAll().stream().map(DtoMapper::mapToDto).collect(Collectors.toList());
        return result;
    }


    @GetMapping(value = "/{id}", consumes = APPLICATION_JSON_VALUE)
    public OfferDto getOffer(@PathVariable("id") long id) {
        OfferDto offerDto = DtoMapper.mapToDto(offerService.findById(String.valueOf(id)));
        return offerDto;
    }


    @PostMapping(value="/create", consumes = APPLICATION_JSON_VALUE)
    public void create(@RequestBody OfferCreateDto offerDto){
        Band band = bandService.findById(offerDto.getBandId());
        Offer offer = CreateDtoMapper.mapToEntity(offerDto);
        offer.setBand(band);
        offerService.create(offer);
    }


    @PutMapping(value = "/update", consumes = APPLICATION_JSON_VALUE)
    public void update(@RequestBody OfferUpdateDto dto) {
        Band band = bandService.findById(dto.getBandId());
        Offer offer = offerService.findById(dto.getId());
        Offer updatedOffer = UpdateDtoMapper.mapToEntity(dto, offer);
        updatedOffer.setBand(band);
        offerService.update(updatedOffer);
    }


    @DeleteMapping(value = "/delete", consumes = APPLICATION_JSON_VALUE)
    public void delete(@RequestBody OfferUpdateDto dto) {
        Offer offer = offerService.findById(dto.getId());
        offerService.delete(offer);
    }
}
