package cz.fi.muni.pa165.contollers;

import cz.fi.muni.pa165.AlbumService;
import cz.fi.muni.pa165.SongService;
import cz.fi.muni.pa165.dto.song.SongCreateDto;
import cz.fi.muni.pa165.dto.song.SongDto;
import cz.fi.muni.pa165.dto.song.SongUpdateDto;
import cz.fi.muni.pa165.entity.Album;
import cz.fi.muni.pa165.entity.Song;
import cz.fi.muni.pa165.mapper.CreateDtoMapper;
import cz.fi.muni.pa165.mapper.DtoMapper;
import cz.fi.muni.pa165.mapper.UpdateDtoMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.Collectors;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;


@CrossOrigin
@RestController
@RequestMapping("/song")
public class SongController {
    private SongService songService;
    private AlbumService albumService;

    @Autowired
    public SongController(SongService songService, AlbumService albumService){
        this.songService = songService;
        this.albumService = albumService;
    }

    @PostMapping(value = "/create", consumes = APPLICATION_JSON_VALUE)
    public void create(@RequestBody SongCreateDto songDto){
        Album album = albumService.findById(songDto.getAlbumId());
        Song song = CreateDtoMapper.mapToEntity(songDto);
        album.addSong(song);
        song.setAlbum(album);
        songService.create(song);
    }

    @GetMapping(value = "/{id}", produces = APPLICATION_JSON_VALUE)
    public SongDto getSong(@PathVariable("id") long id) {
        return DtoMapper.mapToDto(songService.findById(String.valueOf(id)));
    }

    @GetMapping(value = "/{name}", produces = APPLICATION_JSON_VALUE)
    public  List<SongDto> getSong(@PathVariable("name") String name) {
        return songService.findByName(name).stream().map(DtoMapper::mapToDto).collect(Collectors.toList());
    }

    @PutMapping(value = "/update", consumes = APPLICATION_JSON_VALUE)
    public void update(@RequestBody SongUpdateDto dto) {
        Album album = albumService.findById(dto.getAlbumId());
        Song song = songService.findById(dto.getId());
        song.setAlbum(album);
        Song updateSong = UpdateDtoMapper.mapToEntity(dto, song);
        songService.update(updateSong);
    }

    @DeleteMapping(value = "/delete", consumes = APPLICATION_JSON_VALUE)
    public void delete(@RequestBody SongUpdateDto dto) {
        Song song = songService.findById(dto.getId());
        songService.delete(song);
    }

    @GetMapping
    public List<SongDto> getSongs(){
        return songService.findAll().stream().map(DtoMapper:: mapToDto).collect(Collectors.toList());
    }
}
