package cz.fi.muni.pa165;

import cz.fi.muni.pa165.dao.BandDao;
import cz.fi.muni.pa165.entity.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataRetrievalFailureException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author Dávid Želník 469323
 */
@Service
public class BandServiceImpl implements BandService {

    private final BandDao bandRepository;

    @Autowired
    public BandServiceImpl(BandDao bandRepository) {
        this.bandRepository = bandRepository;
    }

    @Override
    @Transactional
    public Band create(Band band) {
        if (band == null) {
            throw new IllegalArgumentException("Band is null");
        }
        return bandRepository.create(band);
    }

    @Override
    @Transactional
    public Band findById(String id) {
        if (id == null) {
            throw new IllegalArgumentException("Id is null");
        }
        return bandRepository.findById(id);
    }

    @Override
    @Transactional
    public List<Band> findByName(String name) {
        if (name == null) {
            throw new IllegalArgumentException("Name is null");
        }
        return bandRepository.findByName(name);
    }

    @Override
    @Transactional
    public Band update(Band band) {
        if (band == null) {
            throw new IllegalArgumentException("Band is null");
        }

        Band bandFromDB = bandRepository.findById(band.getId());
        if (bandFromDB == null) {
            throw new IllegalArgumentException("Band not found in database");
        }

        return bandRepository.update(band);
    }

    @Override
    @Transactional
    public Band delete(Band band) {
        if (band == null) {
            throw new IllegalArgumentException("Band is null");
        }

        Band bandFromDB = bandRepository.findById(band.getId());
        if (bandFromDB == null) {
            throw new IllegalArgumentException("Band not found in database");
        }

        return bandRepository.delete(band);
    }

    @Override
    @Transactional
    public List<Band> findAll() {
        return bandRepository.findAll();
    }

    @Override
    @Transactional
    public void addAlbum(String bandId, Album album){
        Band band = bandRepository.findById(bandId);
        band.addAlbum(album);
        bandRepository.update(band);
    }

    @Override
    @Transactional
    public void addActivity(String bandId, Activity activity){
        Band band = bandRepository.findById(bandId);
        band.addActivity(activity);
        bandRepository.update(band);
    }

    @Override
    @Transactional
    public void acceptOffer(Offer offer, Musician musician){
        if(offer == null){
            throw new IllegalArgumentException("Offer cannot be null");
        }
        if(musician == null){
            throw new IllegalArgumentException("Musician cannot be null");
        }
        Band band = offer.getBand();
        if(musician.isHasBand()){
            band.removeMusician(musician);
        }
        musician.setBand(band);
        musician.setHasBand(true);
        band.addMusician(musician);
        bandRepository.update(band);
    }
}
