import axios from 'axios'

const ALBUM_URL = 'http://localhost:8083/pa165/rest/album'

class AlbumService {
    async create(nameParam, bandIdParam, songsParam){
        const body = {name: nameParam, bandId: bandIdParam, songs: songsParam}
        await this.send("post", "/create", body)
    }

    async update(albumId, nameParam, bandIdParam, songsParam){
        const body = {id:albumId, name: nameParam, bandId: bandIdParam, songs: songsParam}
        await this.send("put", "/update", body)
    }

    async delete(albumId){
        await this.send("delete", "/delete", { id: albumId })
    }

    async getAlbums() {
        return axios.get(ALBUM_URL, {
            headers: {
                'Access-Control-Allow-Origin': '*',
                'Accept': 'application/json'
            }
        });
    }

    async getAlbumsBand(id) {
        const url = `${ALBUM_URL}/albums_band/${id}`
        const response = await axios.get(url, {
            headers: {
                'Access-Control-Allow-Origin': '*',
                'Accept': 'application/json'
            }
        });
        return response.data
    }

    async send(method, url, body){
        await axios({
            headers: {
                'Accept': 'application/json',
                'Access-Control-Allow-Origin': '*',
                'Content-Type': 'application/json',
              },
            method: method,
            url: ALBUM_URL + url,
            data: JSON.stringify(body)
        });
    }
}

export default new AlbumService();